﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;
using System.Threading.Tasks;

namespace AlpacaProject.ControlFlow
{
    public class AggregateExceptionExample
    {

        [UnitTestMethod]
        public void WhatDoesThisCodeDo()
        {
            try
            {
                Parallel.Invoke(
                    () => {
                        int x = 0;
                        int y = 100 / x;    // throws DivideByZeroException
                    }
                    , () => {
                        object p = null;
                        var s = p.ToString();   // throws NullReferenceException
                    });
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
        }

        [UnitTestMethod]
        public void FlattenTest()
        {
            try
            {
                Parallel.Invoke(
                    () => {
                        Parallel.Invoke(
                            () => { throw new NotImplementedException("0.0"); }
                            , () => { throw new NotImplementedException("0.1"); }
                            , () => { throw new NotImplementedException("0.2"); }
                            );
                    }
                    , () => {
                        Parallel.Invoke(
                            () => { throw new NotImplementedException("1.0"); }
                            , () => {
                                Parallel.Invoke(
                                    () => { throw new NotImplementedException("1.1.0"); }
                                    , () => { throw new NotImplementedException("1.1.1"); }
                                    , () => { throw new NotImplementedException("1.1.2"); }
                                    );
                            }
                            , () => { throw new NotImplementedException("1.2"); }
                            );
                    });
            }
            catch (AggregateException ex)
            {
                Console.WriteLine("Before Flatten:");
                foreach (var innerEx in ex.InnerExceptions)
                    Console.WriteLine("   Caught {0} exception: {1}", innerEx.GetType().Name, innerEx.Message);

                AggregateException flatEx = ex.Flatten();
                Console.WriteLine();
                Console.WriteLine("After Flatten");
                foreach (var innerEx in flatEx.InnerExceptions)
                    Console.WriteLine("   Caught {0} exception: {1}", innerEx.GetType().Name, innerEx.Message);
            }
        }

        [UnitTestMethod]
        public void HandleExTest()
        {
            try
            {
                Parallel.Invoke(
                    () => {
                        Parallel.Invoke(
                            () => { throw new NotImplementedException("0.0"); }
                            , () => { throw new NotImplementedException("0.1"); }
                            , () => { throw new NotImplementedException("0.2"); }
                            );
                    }
                    , () => {
                        Parallel.Invoke(
                            () => { throw new NotImplementedException("1.0"); }
                            , () => {
                                Parallel.Invoke(
                                    () => { throw new NotImplementedException("1.1.0"); }
                                    , () => { throw new NotImplementedException("1.1.1"); }
                                    , () => { throw new NotImplementedException("1.1.2"); }
                                    );
                            }
                            , () => { throw new NotImplementedException("1.2"); }
                            );
                    });
            }
            catch (AggregateException ex1)
            {
                try
                {
                    // Let's grab all the exceptions at the 2nd level
                    ex1.Flatten().Handle(innerEx => {
                        if (innerEx.Message.Length == 3)
                        {
                            Console.WriteLine("   Handled {0} excepiton: {1}", innerEx.GetType().Name, innerEx.Message);
                            return true;// Indicate that we handled the exception
                        }
                        else
                        {
                            // if returning false, then the Handle method will throw a new AggregateException
                            return false;
                        }
                    });
                }
                catch (AggregateException ex2)
                {
                    // Let's grab all the exceptions at the 3rd level
                    ex2.Flatten().Handle(innerEx => {
                        if (innerEx.Message.Length == 5)
                        {
                            Console.WriteLine("   Handled {0} excepiton: {1}", innerEx.GetType().Name, innerEx.Message);
                            return true;// Indicate that we handled the exception
                        }
                        else
                        {
                            // if returning false, then the Handle method will throw a new AggregateException
                            return false;
                        }
                    });
                }
            }
        }

    }
}
