﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;
using System.Threading.Tasks;

namespace AlpacaProject
{
    public class ParallelSamples
    {

        [UnitTestMethod]
        public void ParallelFor_vs_ForLoop()
        {
            int n = 1000;

            int[] loopAry = new int[n];
            for (int i = 0; i < n; i++)
            {
                loopAry[i] = i * i;
            }

            int[] parallelAry = new int[n];
            Parallel.For(0, n, i => {
                parallelAry[i] = i * i;
            });

            CollectionAssert.AreEqual(loopAry, parallelAry);
        }

        [UnitTestMethod]
        public void ParallelInvoke_SimpleSlideExample()
        {
            int x = 0;

            Parallel.Invoke(
                () => { x = 1; }
                , () => { x = 2; }
                );

            // Note: This code has a data race but we know that both of the
            // actions have completed so x will be one of the two values.
            Console.WriteLine("x={0}", x);
            Assert.IsTrue(x == 1 || x == 2);
        }

        [UnitTestMethod]
        public void ParallelForEach_IEnumerable()
        {
            // Lets get a series of Fibonacci numbers
            int firstN = 1;
            int count = 20;
            var fibsWithN = Fibonacci.GetSeries(firstN).Take(count);

            Parallel.ForEach(fibsWithN, (fib, _, i) => {
                Console.WriteLine("Fib(n={0}) = {1}", i + firstN, fib);
            });
        }

    }
}
