﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Concurrency.TestTools.UnitTesting;

namespace AlpacaProject
{
    /// <summary>
    /// This test class implements the strategies mentioned in the paper
    /// Patterns of Parallel Programming (PoPP) by Stephen Toub
    /// Section: TOO FINE-GRAINED, TOO COURSE GRAINED
    /// </summary>
    /// <remarks>
    /// http://www.microsoft.com/downloads/details.aspx?FamilyID=86b3d32b-ad26-4bb8-a3ae-c1637026c3ee&displaylang=en
    /// </remarks>
    public class PoPP_MyParallelTests
    {

        [UnitTestMethod]
        public void InvokeAction_OneTaskPerElement()
        {
            int N = 1000;
            int[] data = new int[N];
            MyParallel.InvokeAction_OneTaskPerElement(
                Enumerable.Range(0, N)
                .Select(i => new Action(() => data[i] = i + 1))
                .ToArray()
                );

            // Verify
            for (int i = 0; i < N; i++)
                Assert.AreEqual(i + 1, data[i], "idx[{0}]", i);
        }

        [UnitTestMethod]
        public void InvokeFunc_OneTaskPerElement()
        {
            int N = 1000;
            AssertInvokeFuncCorrect(MyParallel.InvokeFunc_OneTaskPerElement, N);
        }

        [UnitTestMethod]
        public void InvokeFunc_ParentChildTasks()
        {
            int N = 1000;
            AssertInvokeFuncCorrect(MyParallel.InvokeFunc_ParentChildTasks, N);
        }

        [UnitTestMethod]
        public void InvokeFunc_ParallelFor()
        {
            int N = 1000;
            AssertInvokeFuncCorrect(MyParallel.InvokeFunc_ParallelFor, N);
        }

        [UnitTestMethod]
        public void InvokeFunc_Plinq()
        {
            int N = 1000;
            AssertInvokeFuncCorrect(MyParallel.InvokeFunc_Plinq, N);
        }

        [UnitTestMethod]
        public void InvokeFunc_LimitDegreeOfParallelism()
        {
            int N = 1000;
            AssertInvokeFuncCorrect(MyParallel.InvokeFunc_LimitDegreeOfParallelism, N);
        }

        private void AssertInvokeFuncCorrect(MyParallel.InvokeFuncDelegate<int> invokeFunc, int N)
        {
            var funcs = Enumerable.Range(0, N)
                .Select(i => new Func<int>(() => i + 1))
                .ToArray();
            int[] data = invokeFunc(funcs);

            // Verify
            Assert.IsNotNull(data);
            Assert.AreEqual(N, data.Length);
            for (int i = 0; i < N; i++)
                Assert.AreEqual(i + 1, data[i], "idx[{0}]", i);
        }

    }
}
