﻿//--------------------------------------------------------------------------
// 
//  Copyright (c) Microsoft Corporation.  All rights reserved. 
// 
//  File: Raytracer.cs
//
//--------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;

namespace Microsoft.ParallelComputingPlatform.ParallelExtensions.Samples
{
    /// <summary>
    /// A ray tracer implementation that process each pixel on the screen one
    /// at a time.
    /// </summary>
    public sealed class SequentialRaytracer : RayTracerBase
    {

        public SequentialRaytracer(int screenWidth, int screenHeight)
            : base(screenWidth, screenHeight)
        {
        }


        public override void Render(Scene scene, Int32[] rgbBuffer)
        {
            for (int y = 0; y < screenHeight; y++)
            {
                int stride = y * screenWidth;
                Camera camera = scene.Camera;
                Vector cameraPos = camera.Pos;
                for (int x = 0; x < screenWidth; x++)
                {
                    Color color = TraceRay(new Ray(cameraPos, GetPoint(x, y, camera)), scene, 0);
                    rgbBuffer[x + stride] = color.ToInt32();
                }
            }
        }

    }
}
